import numpy as np
from ash import *

num_cores = 16

pdb_nam = "protein_and_ligand.pdb"
frag_prot = Fragment(pdbfile = pdb_nam)

thisdir = ("/home/jzw5/quantum-olfaction-data/2024-06_protein-ligand-data/"
           "isotopes-all/test")

# For now, we're going to try only the ligand as active site.
lig_ind = [4836, 4837, 4838, 4839, 4840, 4841, 4842, 4843, 4844, 4845]
         [C1,   C2,   C3,   O2,   O2,   H21,  H22,  H31,  H32,  H33]

# MM region from OpenMM
# N.B.: I believe CHARMM36 actually means CHARMM36m, see
#   https://github.com/openmm/openmmforcefields/issues/298
# (CHARMM36m is an improved version, see
#   https://doi.org/10.1038/nmeth.4067)
# Creating the topology was a journey; see the associated .md file.
# I think periodic_cell_dimensions is in angstrom
mm = OpenMMTheory(pdbfile = pdb_nam, 
                  GROMACSfiles = True,
                  grofile = "protein_and_ligand.gro",
                  gromacstopdir = thisdir,
                  gromacstopfile = "topol.top",
                  periodic = True,
                  periodic_cell_dimensions = [80.1100, 80.1100, 115.9510, 
                                              90, 90, 90],
                  platform = 'CPU',
                  numcores = num_cores,
                  autoconstraints = None,
                  rigidwater = False)

# QM region with PySCF.
# Basis:      Dunning augmented triple-zeta.
# Functional: B3LYP (reasonable accuracy, efficient, analytic gradient)
#             with D4 dispersion correciton
qm = PySCFTheory(numcores = num_cores,
                 scf_type = 'RKS',
                 basis = 'aug-cc-pVTZ',
                 functional = 'b3lyp',
                 dispersion = 'D4')

# QM/MM object combines
qmmm = QMMMTheory(qm_theory = qm, mm_theory = mm,
                  fragment = frag_prot,
                  qm_charge = -1,
                  embedding = "Elstat",
                  qmatoms = lig_ind,
                  printlevel = 2)

# QM/MM geometry optimization only of the ligand!
active_ind = lig_ind
geom_opt = Optimizer(fragment = frag_prot, theory = qmmm,
                     ActiveRegion = True, actatoms = active_ind,
                     maxiter = 200,
                     charge = -1, mult = 1)

# Vibrational frequency calculation.
#   hessatoms = [active_ind] is a partial Hessian for just the active site!
#   hessatoms_masses = [] specifies the isotopes. Should be length of hessatoms

# Various sub-objects of interest:
#   num_freq.hessian; 
#   num_freq.frequencies (cm**-1) (a NumPy array); 
#   num_freq.IR_intensities (km/mol) (a NumPy array).

# 2024-07-03: Only have to run NumFreq() once; isotope masses are only post-DFT
# Via communication with Ragnar Bjornsson: the NEW branch of ASH now provides
#   new functionality in write_to_disk(); it can read results of NumFreq calc
#   and get frequencies/IR intensities for different masses afterward.
# So we'll do the original calc with the fully hydrogenated masses.
# Let's do the isotope isomers in lexicographic order, thinking of H==0, D==1
def num_to_isotope(n, num_h):
  """ Given integer 0 <= n <= 2**num_h, return corresponding isotopes and masses. """
  if not type(n) is int or n < 0 or not type(num_h) is int or  n > 2**num_h - 1:
    return TypeError("Input integers n, num_h such that  0 <= n <= 2**num_h.")
  
  # Binary representation: 5 digits, no leading '0b'; 0 -> h, 1 -> d
  isotopes = str(bin(n)[2:].zfill(num_h)).replace('0', 'h').replace('1', 'd')

  masses = [12.01, 12.01, 12.01, 16.00, 16.00]
  for atom in isotopes:
    if atom == 'h':
      masses.append(1.008)
    elif atom == 'd':
      masses.append(2.014)
  return {isotopes : masses}

lig_mass = dict()
num_h = 5
for n in range(2**num_h):
  lig_mass.update(num_to_isotope(n, num_h))

print(lig_mass)

# We have to run the vibrational frequency calculation once. 
# Might as well use the all-H case, although we'll rediagonalize the Hessian
# when saving the csv later
#vib_main = NumFreq(fragment = frag_prot, theory = qmmm, npoint = 2,
#                  hessatoms = lig_ind, hessatoms_masses = lig_mass['hhhhh'],
#                  numcores = 1, runmode = 'serial', temp = 310)
#vib_main.write_to_disk(filename = "ASH.result")

result = read_results_from_file(filename = "ASH.result")
print(result)

# From communication with Ragnar Bjornsson, developer of ASH
for iso_label, mass_list in lig_mass.items():

  # Dig out the data from the NumFreq calculation
  hessian = result.hessian
  hesselems=result.freq_elems
  hessatoms=result.freq_atoms
  hesscoords=result.freq_coords
  TRmodenum=result.freq_TRmodenum
  projection=result.freq_projection
  scaling_factor=result.freq_scaling_factor
  dipole_derivs=result.freq_dipole_derivs
  polarizability_derivs=result.freq_polarizability_derivs
  polarizability_derivs = [np.array(i) for i in polarizability_derivs]
  Raman=result.freq_Raman

  # Diagonalize the Hessian
  frequencies, nmodes, evectors, mode_order = \
  ash.modules.module_freq.diagonalizeHessian(hesscoords, hessian, mass_list, 
                                             hesselems, TRmodenum = TRmodenum,
                                             projection = projection)
  
  print("Diagonalization of frequencies complete")
  print("Now scaling frequencies by scaling factor:", scaling_factor)
  frequencies = scaling_factor * np.array(frequencies)

  # Get IR intensities, if dipoles are available
  if dipole_derivs is not None:
    if np.any(dipole_derivs):
      IR_intens_values = \
        ash.modules.module_freq.calc_IR_Intensities(mass_list, evectors,
                                                    dipole_derivs)
    else:
      IR_intens_values = None
  
  # Get Raman activities, if polarizabilities are available
  if Raman is True:
    print("Raman calculation active")
    if polarizability_derivs is None or not np.any(polarizability_derivs):
      print("No polarizability information found! Skipping Raman intensities.")
      Raman_activities = None
      depolarization_ratios = None
    else:
      print("Polarizability derivatives available!")
      Raman_activities, depolarization_ratios = \
        ash.modules.module_freq.calc_Raman_activities(mass_list, evectors,
                                                      polarizability_derivs)
  else:
    Raman_activities = None
    depolarization_ratios = None
  
  # Print the frequency output
  print("type(frequencies) = " + str(type(frequencies)))
  print("type(IR_intens_values) = " + str(type(IR_intens_values)))

  ash.modules.module_freq.printfreqs(frequencies, len(hessatoms),
                                     TRmodenum = TRmodenum, 
                                     intensities = IR_intens_values,
                                     Raman_activities = Raman_activities)
 
  # Get and print out thermochemistry
  thermodict = ash.modules.module_freq.thermochemcalc(frequencies, hessatoms,
                                                      frag_prot, frag_prot.mult)

  the_vib = np.column_stack((frequencies, IR_intens_values))
  np.savetxt("vib_" + iso_label + ".csv", the_vib,
             fmt = '%12.4e', delimiter = ", ",
             header = "  freq_cm-1,  IRint_km/mol", comments = ' ')

