# Creating a protein/ligand topology
One cannot simply dump a protein/small-molecule PDB file into ASH (or OpenMM) and have it run, even if you specify the CHARMM36m force field first. This is because the force field works in terms of _residues_, and it doesn't recognize the ligand.

So we have to create a custom topology. At the recommendation of Ge Song, we follow the [Lemkul Lab's tutorial](http://www.mdtutorials.com/gmx/complex/index.html) on protein-ligand complexes.

An important note: Part of this is done on the cluster (the parts using GROMACS), part on my laptop (the part that requires Avogadro, if only because it's faster than trying to use a remote GUI), and part on the Internet (CGenFF).

## Step 1: The protein topology
Follows [the first step of the tutorial](http://www.mdtutorials.com/gmx/complex/01_pdb2gmx.html). This is all done on the cluster.

First, get the ligand, which is called PPI (propanoic acid; here, it's deprotonated) out of the protein PDB file.
* `cp protein_and_ligand.pdb protein.pdb`;
* `grep 'PPI' protein_and_ligand.pdb > ppi.pdb`;
* `sed -i '/PPI/d' protein.pdb`.

Easy enough. Then we'll make the GROMACS topology file with [a built-in command](https://manual.gromacs.org/current/onlinehelp/gmx-pdb2gmx.html#gmx-pdb2gmx).
* Download [the latest CHARMM36 force field files for GROMACS](http://mackerell.umaryland.edu/charmm_ff.shtml#gromacs) to the working directory. Note that it already includes the CHARMM36m modifications. This is a `.tar.gz` file, so unzip it with `tar-zxvf charmm36-jul2022.ff.tgz`.
* Load GROMACS on the cluster with `module load gromacs/5.1.4` (as of 2024-05-31, the most recent version apart from the GPU one).
* Then use GROMACS' `pdb2gmx` utility: `gmx pdb2gmx -f protein.pdb -o protein.gro -water none -ff charmm36-jul2022`.
    *  `-water none` option indicates no water model (we'd need one if we were including the water in the original PDB, of course!)
    * `-ff charmm36-jul2022` chooses the force field in the current directory, if it is named `charmm36-jul2022.ff`.
    * We use the default condition (ionized) for the terminal amino acids.

This gives us:
* `protein.gro`, the GROMACS coordinate file;
* `topology.top`, the topology file; and also
* `posre.itp`, which constrains all the heavy atoms from the original PDB file from moving.

## Step 2: The ligand topology
Follows [the second step of the tutorial](http://www.mdtutorials.com/gmx/complex/02_topology.html), and is done on my laptop and the Web.

Download `ppi.pdb`, the ligand PDB file (it's okay that it doesn't have the first/last lines with the column labels and `END`). Open it [in Avogadro](https://avogadro.cc/), and `File > Save As...` to the Sybyl Mol2 format `ppi.mol2`. 

#### ALTERNATIVELY, THIS CAN BE DONE ON THE CLUSTER!
Create the Python virtual environment up here instead (see below, in the "Converting to GROMACS format" section). Then run `mamba install openbabel` to obtain the OpenBabel command-line tool for converting between chemical file formats.

Run `obabel ppi.pdb -O ppi.mol2` to convert from PDB to Mol2 format. (Or, `obabel -ipdb ppi.pdb -omol2 > ppi.mol2` works as well.)

In `ppi.mol2`, replace the `*****` in the second line by the molecule name, `PPI`. Then download the Perl script `sort_mol2_bonds.pl` available in the second step website to the working directory, and run it:
* `perl sort_mol2_bonds.pl ppi.mol2 ppi_fix.mol2`.
This corrects the order of the bonds, which are randomized by Avogadro.

Now we will create the CHARMM topology for the ligand using the [official website, CGenFF](https://cgenff.com/). Start by creating an account, or logging in if you have already. (It's free for academics.)
* Upload `ppi_fix.mol2` with the drag-and-drop window.
* No need to change any of the default options (you could change the name if you wanted); proceed to `Process Input`.
* Click `Run CgenFF Engine`, then `Download CGenFF Results`; uncompress the resulting `ppi_fix.zip` in the working directory.

This gives us a topology file, `ppi_fix.str`, but it is in CHARMM format: we need to convert it to GROMACS. (First, look at `ppi_fix.str` with a text editor; ensure that the penalties for charges and new dihedral parameters are low, and that the charges are as expected.)

Converting to GROMACS format requires _specific versions_ of some Python packages; it's best to create a new environment for it. Here's a refresher, using `mamba` (you can replace `mamba` by `conda` with the same effect; `mamba` is the same thing but without the initial set of packages).
* Create the new environment; here we'll call it `gromacs_topol`: `mamba create -n gromacs_topol`;
* Switch to it: `mamba activate gromacs_topol`;
* Download Python (version 3.7), NetworkX (version 2.3), and NumPy: `mamba install python=3.7; mamba install networkx=2.3; mamba install numpy`, and answer Y(es) to the installation prompts.

Now, with the `gromacs_topol` environment still active:
* Download the file `cgenff_charmm2gmx_py3_nx2.py` from [its GitHub webpage](https://github.com/Lemkul-Lab/cgenff_charmm2gmx) to your working directory;
* Copy the CHARMM36 force field directory from the cluster to this working directory;
* Run the script: `python cgenff_charmm2gmx.py PPI ppi_fix.mol2 ppi_fix.str charmm36-jul2022.ff`.

The output is reproduced here:
```
(gromacs_topol) user@LAPTOP cwd % python cgenff_charmm2gmx_py3_nx2.py PPI ppi_fix.mol2 ppi_fix.str charmm36-jul2022.ff

NOTE 1: Code tested with Python 3.5.2 and 3.7.3. Your version: 3.7.16 (default, Jan 17 2023, 09:28:58) 

[Clang 14.0.6 ]

NOTE 2: Code tested with NetworkX 2.3. Your version: 2.3

NOTE 3: Please be sure to use the same version of CGenFF in your simulations that was used during parameter generation:

--Version of CGenFF detected in  ppi\_fix.str : 4.6

--Version of CGenFF detected in  charmm36-jul2022.ff/forcefield.doc : 4.6

NOTE 4: To avoid duplicated parameters, do NOT select the 'Include parameters that are already in CGenFF' option when uploading a molecule into CGenFF.

============ DONE ============

Conversion complete.

The molecule topology has been written to ppi.itp

Additional parameters needed by the molecule are written to ppi.prm, which needs to be included in the system .top

PLEASE NOTE: If your topology has lone pairs, you must use GROMACS version 2020 or newer to use 2fd construction

Older GROMACS versions WILL NOT WORK as they do not support 2fd virtual site construction

============ DONE ============
```

The "2fd construction" [means](https://manual.gromacs.org/current/reference-manual/topologies/particle-type.html) virtual sites on the line between two atoms at a fixed distance. There are lone pairs in PPI, and the version (5.1.4) of GROMACS we're using to create the protein topology is rather older than 2020, but since we don't actually need to do MD I think we're okay.

Anyway, this created a few files:
* `ppi.itp`, the topology for the ligand;
* `ppi.top`, its wrapper;
* `ppi.prm`, the custom parameters for the force field;
* `ppi_ini.pdb`, which is the ligand PDB file with its atom names matching the topology.

Copy `ppi.itp`, `ppi.prm`, and `ppi_ini.pdb` to the cluster.

## Step 3: Putting it together
We're still in [the second step of the tutorial](http://www.mdtutorials.com/gmx/complex/02_topology.html), but we've moved back to the cluster. Make sure GROMACS is still loaded! (`module load gromacs/5.1.4`)

Convert the PPI PDB to `.gro` format: `gmx editconf -f ppi_ini.pdb -o ppi.gro`.

Remember: in step 1, we created the protein-only `protein.gro`. Copy this to a new `protein_and_ligand.gro`. Then, copy the coordinates from `ppi.gro` (the lines starting with `1PPI`) into `protein_and_ligand.gro` _after_ the last line of protein coordinates (in this case a line starting with `305MET`) but before the last line, which defines the simulation box. Increment the second line of `protein_and_ligand.gro`, which counts the total number of atoms, to the new total (`4846`).

Now we need to combine: the overall topology needs to be in only one file, `topol.top`. 
* Insert a line after the position restraint to include the ligand topology: `#include "ppi.itp"`.
* Insert a line for the new dihedral parameters at the _top_ of the file (after the forcefield parameters); it must be _before_ any entry for `[ moleculetype ]`. It looks like this: `#include ppi.prm`.
* In the `[ molecules ]` section at the end, below the line with the protein chin, add `PPI        1`.

That's it! We have created, it is to be hoped, a consistent topology for the protein and the ligand together. These are the files (and directory) we'll need in the ASH working directory:
* `charmm36-jul2022.ff/`, the main force field;
* `protein_and_ligand.gro`, the combined coordinates;
* `topol.top`, the main topology wrapper;
* `ppi.prm`, the extra ligand force field parameters;
* `ppi.itp`, the ligand toplogy;
* `posre.itp`, the position restraints.